
import { createRoot } from 'react-dom/client'
import App from './App.tsx'
import './index.css'
import './styles/mobile-optimizations.css'

// Add global type declaration for gtag
declare global {
  interface Window {
    gtag: (...args: any[]) => void;
    dataLayer: any[];
  }
}

// Mark the start of the app boot
if ('performance' in window && 'mark' in window.performance) {
  window.performance.mark('app_boot_start');
}

// Detectar si estamos en móvil para optimizaciones
const isMobileDevice = window.innerWidth < 768 || 
                       /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);

// Critical path optimization: Load only what's necessary for LCP
const initializeApp = () => {
  // Create root with concurrent mode
  createRoot(document.getElementById("root")!).render(<App />);
  
  // Mark when the app becomes interactive
  if ('performance' in window && 'mark' in window.performance) {
    window.performance.mark('app_interactive');
    
    // Measure time to interactive
    if ('measure' in window.performance) {
      window.performance.measure('time_to_interactive', 'app_boot_start', 'app_interactive');
      const timeToInteractive = window.performance.getEntriesByName('time_to_interactive')[0];
      console.log('Time to interactive:', timeToInteractive.duration);
    }
  }
  
  // Execute non-critical operations with device-specific timing
  const deferTimeout = isMobileDevice ? 5000 : 3000; // Mayor timeout en móvil, y aún más largo que antes
  
  // Execute non-critical operations only after app is interactive and user has had a chance to interact
  if ('requestIdleCallback' in window) {
    (window as any).requestIdleCallback(deferNonCriticalOperations, { timeout: deferTimeout });
  } else {
    // Fallback for browsers that don't support requestIdleCallback
    setTimeout(deferNonCriticalOperations, deferTimeout);
  }
};

// Preload only the most critical font
const preloadCriticalFonts = () => {
  // En móvil, cargar solo una variante de la fuente principal para mejor rendimiento inicial
  const fontUrl = 'https://fonts.googleapis.com/css2?family=Inter:wght@400&display=swap';
  
  // Check if this font is already loaded
  if (document.querySelector(`link[href="${fontUrl}"]`)) {
    return;
  }
  
  const linkEl = document.createElement('link');
  linkEl.rel = 'preload';
  linkEl.href = fontUrl;
  linkEl.as = 'style';
  linkEl.crossOrigin = 'anonymous';
  document.head.appendChild(linkEl);
  
  // Also add link rel="stylesheet" to actually use the font
  // This ensures the font is used immediately
  const styleEl = document.createElement('link');
  styleEl.rel = 'stylesheet';
  styleEl.href = fontUrl;
  styleEl.crossOrigin = 'anonymous';
  document.head.appendChild(styleEl);
};

// Only preload the most critical image on mobile
const preloadCriticalImages = () => {
  // Only load critical images and skip others for better initial load
  if (isMobileDevice) {
    // Skip preloading images on mobile for better initial load time
    return;
  }
  
  const imageUrl = '/angel-bot.png';
  
  const linkEl = document.createElement('link');
  linkEl.rel = 'preload';
  linkEl.href = imageUrl;
  linkEl.as = 'image';
  linkEl.fetchPriority = 'high';
  document.head.appendChild(linkEl);
};

// Defer non-critical operations
const deferNonCriticalOperations = () => {
  // Optimizaciones específicas para móvil
  if (isMobileDevice) {
    // Disable all background animations on mobile to save battery
    document.documentElement.classList.add('reduced-motion');
    document.querySelectorAll('.animate-pulse, .animate-spin, .animate-bounce').forEach(el => {
      el.classList.add('animation-paused');
    });
    
    // Reduce image quality on mobile
    document.querySelectorAll('img:not([data-critical="true"])').forEach(img => {
      if (img instanceof HTMLImageElement) {
        // Add loading="lazy" to all non-critical images
        img.loading = 'lazy';
        // Lower quality versions of images
        if (img.src.includes('.jpg') || img.src.includes('.png')) {
          img.style.filter = 'blur(0.5px)';
        }
      }
    });
    
    // Load additional fonts only when needed
    const addFontEl = document.createElement('link');
    addFontEl.rel = 'stylesheet';
    addFontEl.href = 'https://fonts.googleapis.com/css2?family=Inter:wght@700&display=swap';
    document.head.appendChild(addFontEl);
  } else {
    // Load additional fonts for desktop
    const additionalFontUrl = 'https://fonts.googleapis.com/css2?family=Inter:wght@500;600;700&display=swap';
    const linkEl = document.createElement('link');
    linkEl.rel = 'stylesheet';
    linkEl.href = additionalFontUrl;
    linkEl.crossOrigin = 'anonymous';
    document.head.appendChild(linkEl);
  }
  
  // Skip prefetching on slow connections
  if (document.visibilityState === 'visible' && 
      (window.navigator as any).connection && 
      !(window.navigator as any).connection.saveData && 
      (window.navigator as any).connection.effectiveType !== 'slow-2g') {
    
    // Only prefetch the most important page
    const prefetchLinks = [
      '/love-calculator'
    ];
    
    prefetchLinks.forEach(link => {
      const linkEl = document.createElement('link');
      linkEl.rel = 'prefetch';
      linkEl.href = link;
      document.head.appendChild(linkEl);
    });
  }
};

// Add event listener to mark when the app is interactive
const markInteractive = () => {
  // Report to analytics or performance monitoring
  if (window.performance && window.performance.mark) {
    window.performance.mark('user_interaction');
  }
  
  // Clean up the listener
  document.removeEventListener('click', markInteractive);
  document.removeEventListener('keydown', markInteractive);
};

// Execute critical path in the right order
preloadCriticalFonts();
preloadCriticalImages();
initializeApp();

// Listen for first interaction
document.addEventListener('click', markInteractive);
document.addEventListener('keydown', markInteractive);
